from datetime import datetime
import logging, sys
from lxml import etree
from pprint import pprint

import requests

from lobbyfacts.data import sl, etl_engine, live_engine

from lobbyfacts.data.load.util import upsert_entity
from lobbyfacts.model.representative import *
from lobbyfacts.core import app
from lobbyfacts.model import Accreditation
from lobbyfacts.model import Person


log = logging.getLogger(__name__)

URL = "http://ec.europa.eu/transparencyregister/public/consultation/statistics.do?action=getLobbyistsXml&fileType=ACCREDITED_PERSONS"
_NS = "http://ec.europa.eu/transparencyregister/accreditedPerson/V1"
NS = '{%s}' % _NS


def upsert_person(data):
    entity = upsert_entity(data.get('canonical_name'), data.get('name'))
    data['entity'] = entity
    person = Person.by_person_name(data['first_name'],data['last_name'])
    if person is None:
        log.info("creating new person")
        person = Person.create(data)
    # else:
    #     person.update(data)
    return person

def dateconv(ds):
    return datetime.strptime(ds[0:10], "%Y-%m-%d")
    # return datetime.strptime(ds, "%Y-%m-%d+%H:%M")

def otherdateconv(ds):
    return datetime.strptime(ds.split("+")[0].strip(), "%Y-%m-%dT%H:%M:%S.%f")

def parse(data):
    doc = etree.fromstring(data.encode('utf-8'))
    e_date = doc.findtext('.//' + NS + 'exportDate')
    for ap_el in doc.findall('.//' + NS + 'accreditedPerson'):
        ap = {
            'org_identification_code': ap_el.findtext(NS + 'orgIdentificationCode'),
            #'number_of_ir': ap_el.findtext(NS + 'numberOfIR'),
            'org_name': ap_el.findtext(NS + 'orgName'),
            'title': ap_el.findtext(NS + 'title'),
            'first_name': ap_el.findtext(NS + 'firstName'),
            'last_name': ap_el.findtext(NS + 'lastName'),
            'start_date': dateconv(ap_el.findtext(NS + 'accreditationStartDate')),
            'end_date': dateconv(ap_el.findtext(NS + 'accreditationEndDate')),
            'extract_date': otherdateconv(e_date)
            }
        yield ap

def save(person, engine):
    person_obj = {}
    person_obj['title'] = person['title']
    person_obj['first_name'] = person['first_name']
    person_obj['last_name'] = person['last_name']
#    person_obj['position'] = person['position']

    person_list = [person_obj['title'],person_obj['first_name'],person_obj['last_name']]
    person_obj['name'] = u' '.join(item for item in person_list if item)
    return_val = False

    rep = Representative.by_identification_code(person['org_identification_code'])
    if(rep is None):
        log.warn("Cannot associate with a registered interest: %r", person)
    else:
    	data_ = {}
        person_obj = upsert_person(person_obj)
        data_['person'] = person_obj
        data_['representative'] = rep
        data_['start_date'] = person['start_date']
        data_['end_date'] = person['end_date']
        data_['extract_date'] = person['extract_date']
        data_['status'] = 'active'
        accreditation = Accreditation.by_all(data_)
        if accreditation is None:
            log.info("creating new accreditation")
            accreditation = Accreditation.create(data_)
        else:
            accreditation.status = "active"
        return_val = accreditation
        # else:
        #     accreditation.update(data_)
    db.session.commit()
    return return_val

def extract_data(engine, data):
    log.info("Extracting accredditation data...")
    sl.update(engine, 'accreditation', {}, {'status': 'inactive'}, ensure=False)

    line = ""
    try:
        f = open('/home/lobbyfacts/api-newxml/bin/crucialfile_acc','r')
        line = f.readline()
        f.close()
    except:
        log.error("something went wrong reading the acc file")

    acc_list = line.split(",")
    done_list = []
    new_acc_set = set()

    for i, ap in enumerate(parse(data)):
        return_val = save(ap, engine)
        if return_val != False:
            accreditation = return_val
            new_acc_set.add(accreditation.id)
            if(accreditation.id not in acc_list):
                if accreditation.id not in done_list:
                    log.info(accreditation.id + " acc has appeared.")
                    _data = {'accreditation_id': accreditation.id,'date':ap['extract_date']}
                    rapp = acc_appearance.create(_data)
            else:
                acc_list.remove(accreditation.id)

            done_list.append(accreditation.id)
        if i % 100 == 0:
            log.info("Extracted: %s...", i)


    for tacc in acc_list:
        # add entry in DB to signal rep disappearance
        if tacc != "" and tacc is not None:
            rapp_id = db.session.execute("select id from acc_appearance where accreditation_id = :rid and disappeared_date is null",{'rid':tacc})
            t_rapp = rapp_id.fetchone()
            if t_rapp is None:
                log.error("rep without appearance date disappeared. not good: %s", tacc)
            else:
                db.session.execute("update acc_appearance set disappeared_date = :dd where id = :tid",{'dd':ap['extract_date'],'tid':t_rapp['id']})
                log.info(tacc + " acc has disappeared.")

    newline = ",".join(new_acc_set)
    try:
        f = open('/home/lobbyfacts/api-newxml/bin/crucialfile_acc','w')
        f.write(newline)
        f.close()
    except:
        log.error("something went wrong writing to acc file.")

def extract(engine):
    res = requests.get(URL)
    extract_data(engine, res.content.decode('utf-8'))

def extract_new():
    engine = live_engine()
    res = requests.get(URL)
    extract_data(engine, res.content.decode('utf-8'))

def external_url_handler(error, endpoint, values):
    return ''

if __name__ == '__main__':
    app.url_build_error_handlers.append(external_url_handler)
    ctx = app.test_request_context()
    ctx.push()
    global request
    request = app.preprocess_request()
    if len(sys.argv)>=2:
        engine = live_engine()
        log.info("Processing file " + sys.argv[1])
        with open(sys.argv[1],'r') as fd:
            extract_data(engine, fd.read().decode('utf-8'))
    else:
        log.error("Argument expected: xml file.")
